@@ -98,14 +98,16 @@ instance TimeAxis LocalTime
 
 data (TimeAxis t) => ChartKind t = KindEvent
                | KindDuration  { mapName :: S.ByteString -> S.ByteString, subKind :: ChartKind t }
-               | KindHistogram { binSize :: Delta t }
+               | KindCount     { binSize :: Delta t }
                | KindQuantile  { binSize :: Delta t, quantiles :: [Double] }
                | KindBinFreq   { binSize :: Delta t, delims    :: [Double] }
-               | KindBinCount  { binSize :: Delta t, delims    :: [Double] }
+               | KindBinHist   { binSize :: Delta t, delims    :: [Double] }
                | KindFreq      { binSize :: Delta t, style :: PlotBarsStyle }
-               | KindCount     { binSize :: Delta t, style :: PlotBarsStyle }
+               | KindHistogram { binSize :: Delta t, style :: PlotBarsStyle }
                | KindLines
                | KindDots
+               | KindCumSum
+               | KindSum       { binSize :: Delta t }
                | KindNone
 
 
@@ -125,7 +127,7 @@ data Conf = forall t . TimeAxis t =>
 
 readConf :: [String] -> Conf
 readConf args = case (words $ single "time format" "-tf" ("date %Y-%m-%d %H:%M:%OS")) of
-    ["num"]  -> readConf' (fmap (int2double *** id) . B.readInt)
+    ["num"]  -> readConf' readDouble
     "date":f -> readConf' (strptime (B.pack $ unwords f))
     _        -> error "Unrecognized time format (-tf)"
   where
@@ -161,17 +163,19 @@ readConf args = case (words $ single "time format" "-tf" ("date %Y-%m-%d %H:%M:%
         fromTime    = fst `fmap` (parseTime . B.pack $ single "minimum time (inclusive)" "-fromTime" "")
         toTime      = fst `fmap` (parseTime . B.pack $ single "maximum time (exclusive)" "-toTime"   "")
 
-        parseKind ["hist",    n  ] = KindHistogram {binSize=read n}
+        parseKind ["count",   n  ] = KindCount     {binSize=read n}
         parseKind ["freq",    n  ] = KindFreq      {binSize=read n,style=BarsClustered}
         parseKind ["freq",    n,s] = KindFreq      {binSize=read n,style=parseStyle s}
-        parseKind ["count",   n  ] = KindCount     {binSize=read n,style=BarsClustered}
-        parseKind ["count",   n,s] = KindCount     {binSize=read n,style=parseStyle s}
+        parseKind ["hist",    n  ] = KindHistogram {binSize=read n,style=BarsClustered}
+        parseKind ["hist",    n,s] = KindHistogram {binSize=read n,style=parseStyle s}
         parseKind ["event"       ] = KindEvent
         parseKind ["quantile",b,q] = KindQuantile  {binSize=read b, quantiles=read ("["++q++"]")}
         parseKind ["binf",    b,q] = KindBinFreq   {binSize=read b, delims   =read ("["++q++"]")}
-        parseKind ["binc",    b,q] = KindBinCount  {binSize=read b, delims   =read ("["++q++"]")}
+        parseKind ["binh",    b,q] = KindBinHist  {binSize=read b, delims   =read ("["++q++"]")}
         parseKind ["lines"       ] = KindLines
         parseKind ["dots"        ] = KindDots
+        parseKind ["cumsum"      ] = KindCumSum
+        parseKind ["sum",     b  ] = KindSum       {binSize=read b}
         parseKind ("duration":ws)  = KindDuration  {subKind=parseKind ws, mapName=id}
         parseKind (('d':'u':'r':'a':'t':'i':'o':'n':'[':sep:"]"):ws)  
                                    = KindDuration  {subKind=parseKind ws, mapName = fst . S.break (==sep)}
@@ -251,15 +255,17 @@ makeChart chartKindF events@((t0,_,_):_) = renderLayout1sStacked plots
     plotTrack      name es = plotWithKind name (chartKindF name) es
 
     plotWithKind   name k es = case k of
-      KindHistogram bs    -> withAnyOrdinate $ plotTrackHistogram name es bs
+      KindCount     bs    -> withAnyOrdinate $ plotTrackCount     name es bs
       KindFreq      bs k  -> withAnyOrdinate $ plotTrackFreq      name es bs k
-      KindCount     bs k  -> withAnyOrdinate $ plotTrackCount     name es bs k
+      KindHistogram bs k  -> withAnyOrdinate $ plotTrackHist      name es bs k
       KindEvent           -> withAnyOrdinate $ plotTrackEvent     name es
       KindQuantile  bs qs -> withAnyOrdinate $ plotTrackQuantile  name es qs bs
       KindBinFreq   bs vs -> withAnyOrdinate $ plotTrackBinFreqs  name es vs bs
-      KindBinCount  bs vs -> withAnyOrdinate $ plotTrackBinCounts name es vs bs
+      KindBinHist   bs vs -> withAnyOrdinate $ plotTrackBinHist   name es vs bs
       KindLines           -> withAnyOrdinate $ plotTrackLines     name es
       KindDots            -> withAnyOrdinate $ plotTrackDots      name es
+      KindSum       bs    -> withAnyOrdinate $ plotTrackSum       name es bs
+      KindCumSum          -> withAnyOrdinate $ plotTrackCumSum    name es
       KindDuration  _ _   -> error "KindDuration should not be plotted"
       KindNone            -> error "KindNone should not be plotted"
 
@@ -275,12 +281,12 @@ makeChart chartKindF events@((t0,_,_):_) = renderLayout1sStacked plots
                   plot_bars_alignment ^= BarsLeft     $
                   defaultPlotBars
 
-    plotTrackHistogram name es bs = layoutWithTitle (plotBars plot) name
+    plotTrackCount name es bs = layoutWithTitle (plotBars plot) name
       where plot = plot_bars_values  ^= barsData $ ourPlotBars
             barsData = [(t,[n]) | ((t,_),n) <- edges2bins bs t0 (edges es)]
 
     plotTrackFreq  = plotTrackAtoms atoms2freqs
-    plotTrackCount = plotTrackAtoms atoms2counts
+    plotTrackHist  = plotTrackAtoms atoms2hist
 
     plotTrackAtoms f name es bs k = layoutWithTitle (plotBars plot) name
       where plot = plot_bars_style       ^= k           $
@@ -321,9 +327,9 @@ makeChart chartKindF events@((t0,_,_):_) = renderLayout1sStacked plots
       where
         vals = byTimeBins ((0:).values2binFreqs  vs) bs t0 (values es)
         n    = length vs
-    plotTrackBinCounts name es vs bs = plotTrackBars vals (binTitles vs) name (binColor n)
+    plotTrackBinHist   name es vs bs = plotTrackBars vals (binTitles vs) name (binColor n)
       where
-        vals = byTimeBins ((0:).values2binCounts vs) bs t0 (values es)
+        vals = byTimeBins ((0:).values2binHist vs) bs t0 (values es)
         n    = length vs
 
     plotTrackBars :: (BarsPlotValue a) => [(t,[a])] -> [String] -> S.ByteString -> (Int -> AlphaColour Double) -> Layout1 t a
@@ -339,14 +345,20 @@ makeChart chartKindF events@((t0,_,_):_) = renderLayout1sStacked plots
     toBars tvs = [(t,diffs vs) | (t,vs) <- tvs]
     diffs xs = zipWith (-) xs (0:xs)
 
-    plotTrackLines name es = layoutWithTitle (toPlot plot) name
-      where plot = plot_lines_values ^= [values es] $ defaultPlotLines
+    plotLines name vs = layoutWithTitle (toPlot plot) name
+      where plot = plot_lines_values ^= [vs] $ defaultPlotLines
 
+    plotTrackLines name es = plotLines name (values es)
+    
     plotTrackDots  name es = layoutWithTitle (toPlot plot) name
       where plot = plot_points_values ^= values es $ 
                    plot_points_style  ^= hollowCircles 4 1 (opaque blue) $ 
                    defaultPlotPoints
 
+    plotTrackCumSum name es = plotLines name $ scanl1 (\(t1,s) (t2,v) -> (t2,s+v)) (values es)
+
+    plotTrackSum name es bs = plotLines name $ byTimeBins sum bs t0 (values es)
+
     layoutWithTitle plot name =
         layout1_title ^= "" $
         layout1_plots ^= [Left plot] $
@@ -434,25 +446,25 @@ getQuantiles qs = \xs -> quantiles' (sort xs)
           | True = x:index (i':is) (j+1) xs
 
 values2binFreqs :: (Ord a) => [a] -> [a] -> [Double]
-values2binFreqs bins xs = map toFreq $ values2binCounts bins xs
+values2binFreqs bins xs = map toFreq $ values2binHist bins xs
   where
     n = length xs
     toFreq = if n==0 then const 0 else (\k -> fromIntegral k/fromIntegral n)
-values2binCounts bins xs = values2binCounts' bins $ sort xs
+values2binHist bins xs = values2binHist' bins $ sort xs
   where
-    values2binCounts' []     xs = [length xs]
-    values2binCounts' (a:as) xs = length xs0 : values2binCounts' as xs'
+    values2binHist' []     xs = [length xs]
+    values2binHist' (a:as) xs = length xs0 : values2binHist' as xs'
       where (xs0,xs') = span (<a) xs
 
-atoms2counts :: (Ord a) => [a] -> [a] -> [Int]
-atoms2counts as xs = map (maybe 0 id . (`M.lookup` m)) as
+atoms2hist :: (Ord a) => [a] -> [a] -> [Int]
+atoms2hist as xs = map (maybe 0 id . (`M.lookup` m)) as
   where
     m          = foldl' insert M.empty xs
     insert m a = M.alter (Just . maybe 1 inc) a m
     inc    n   = n `seq` (n+1)
 
 atoms2freqs :: (Ord a) => [a] -> [a] -> [Double]
-atoms2freqs as xs = map toFreq (atoms2counts as xs)
+atoms2freqs as xs = map toFreq (atoms2hist as xs)
   where
     n = length xs
     toFreq = if n==0 then const 0 else (\k -> fromIntegral k/fromIntegral n)
@@ -472,8 +484,8 @@ showHelp = mapM_ putStrLn [ "",
   "              x is only available if you installed timeplot with --flags=gtk",
   "  -or       - output resolution (default 640x480)",
   "  -if IFILE - input file; '-' means 'read from stdin'",
-  "  -tf TF    - time format: 'num' means that times are integer numbers less than 2^31",
-  "              (for instance, line numbers); 'date PATTERN' means that times are dates",
+  "  -tf TF    - time format: 'num' means that times are floating-point numbers",
+  "              (for instance, seconds elapsed since an event); 'date PATTERN' means that times are dates",
   "              in the format specified by PATTERN - see http://linux.die.net/man/3/strptime,",
   "              for example, [%Y-%m-%d %H:%M:%S] parses dates like [2009-10-20 16:52:43]. ",
   "              We also support %OS for fractional seconds (i.e. %OS will parse 12.4039 or 12,4039).",
@@ -502,27 +514,25 @@ showHelp = mapM_ putStrLn [ "",
   "  'duration[C] XXXX' - same as 'duration', but of a track's name we only take the part before character C.",
   "     For example, if you have processes named 'MACHINE-PID' (i.e. UNIT027-8532) say 'begin something' / ",
   "     'end something' and you're interested in the properties of per-machine durations, use duration[-].",
-  "     An interesting combination is, for example, duration[-] cumsum - you'll see total durations of the",
-  "     something for all pids on a machine, for each machine.",
-  "  'hist N' is for histograms: a histogram is drawn with granularity of N time units, where",
-  "     the bin corresponding to [t..t+N) has value 'what was the maximal number of active events",
+  "  'count N' is for activity counts: a histogram is drawn with granularity of N time units, where",
+  "     the bin corresponding to [t..t+N) has value 'what was the maximal number of active events or impulses",
   "     in that interval'.",
   "  'freq N [TYPE]' is for event frequency histograms: a histogram of type TYPE (stacked or ",
-  "     clustered, default clustered) is drawn for each time bin of size N, about the distribution ",
+  "     clustered, default clustered) is drawn for each time bin of size N, about the *frequency* ",
   "     of various ` events",
-  "  'count N [TYPE]' is for event frequency histograms: a histogram of type TYPE (stacked or ",
-  "     clustered, default clustered) is drawn for each time bin of size N, about the counts of ",
+  "  'count N [TYPE]' is for event count histograms: a histogram of type TYPE (stacked or ",
+  "     clustered, default clustered) is drawn for each time bin of size N, about the *counts* of ",
   "     various ` events",
   "  'quantile N q1,q2,..' (example: quantile 100 0.25,0.5,0.75) - a bar chart of corresponding",
   "     quantiles in time bins of size N",
-  "  'binf N v1,v2,..' (example: binf 100 1,2,5,10) - a bar chart of frequency of values falling",
+  "  'binf N v1,v2,..' (example: binf 100 1,2,5,10) - a histogram of frequency of values falling",
   "     into bins min..v1, v1..v2, .., v2..max in time bins of size N",
-  "  'binc N v1,v2,..' (example: binf 100 1,2,5,10) - a bar chart of counts of values falling",
+  "  'binh N v1,v2,..' (example: binf 100 1,2,5,10) - a histogram of counts of values falling",
   "     into bins min..v1, v1..v2, .., v2..max in time bins of size N",
-  "  'lines' - a simple line plot of numeric values",
-  "  'dots'  - a simple dot plot of numeric values",
-  "  'sum N' - a line plot of the sums of numeric values in time bins of size N",
-  "  'cumsum' - a line plot of the cumulative sums of numeric values seen so far",
+  "  'lines'  - a simple line plot of numeric values",
+  "  'dots'   - a simple dot plot of numeric values",
+  "  'cumsum' - a simple line plot of the sum of the numeric values",
+  "  'sum N'  - a simple line plot of the sum of the numeric values in time bins of size N",
   "N is measured in units or in seconds."
   ]
 
